@@ -4,7 +4,7 @@ import sql
 def ds_define():
     import sys
     from pythk.types import is_subclass, is_lambda
-    frame = sys._getframe(1)
+    frame = sys._getframe(2)
     tables = [(key, value) for key, value in frame.f_locals.items() if is_subclass(value, sql.table)]
     vview_gen = [(key, value) for key, value in frame.f_locals.items() if is_lambda(value)]
     ds_queries = [(key, value) for key, value in frame.f_locals.items() if isinstance(value, ds_query)]
@@ -15,6 +15,7 @@ class ds_query(object):
     pass
 
 
+# This class is supposed to be used as a decorator.
 class vquery(ds_query):
     def __init__(self, vview_gen):
 	self.vview_gen = vview_gen
@@ -57,6 +58,11 @@ class squery(ds_query):
     pass
 
 
+# This function is supposed to be used as a decorator.
+def squery_fs(*field_names):
+    return lambda vview_gen: squery(field_names, vview_gen)
+
+
 class mquery(ds_query):
     def __init__(self, field_names, vview_gen):
 	self.field_names = field_names
@@ -79,12 +85,19 @@ class mquery(ds_query):
     pass
 
 
+# This function is supposed to be used as a decorator.
+def mquery_fs(*field_names):
+    return lambda vview_gen: mquery(field_names, vview_gen)
+
+
 class data_src_meta(type):
     def __new__(M, name, bases, dict):
 	try:
 	    ds_definition = dict['ds_definition']
 	except KeyError:
-	    pass
+            all_tbs = [v for v in dict.values()
+                       if isinstance(v, type) and issubclass(v, sql.table)]
+            dict['_all_tbs'] = all_tbs
 	else:
 	    tables, vview_gens, ds_queries = ds_definition()
 	    all_tbs = []
@@ -157,8 +170,8 @@ class data_src(object):
     
     def insert(self, table, **kws):
 	cu = self.get_cursor()
-	cmd = apply(table.factory, (), kws).gen_insert_cmd()
-	cu.execute(cmd)
+	cmd, args = apply(table.factory, (), kws).gen_insert_cmd()
+	cu.execute(cmd, args)
 	return cu
     
     def update(self, table, **kws):
@@ -172,14 +185,14 @@ class data_src(object):
 	    pass
 	values_o = apply(table.factory, (), kws)
 	
-	cmd = values_o.gen_update_cmd(cond)
-	cu.execute(cmd)
+	cmd, args = values_o.gen_update_cmd(cond)
+	cu.execute(cmd, args)
 	return cu
     
     def delete(self, table, where=None):
 	cu = self.get_cursor()
-	cmd = table.gen_delete_cmd(where)
-	cu.execute(cmd)
+	cmd, args = table.gen_delete_cmd(where)
+	cu.execute(cmd, args)
 	return cu
     
     def get_cursor(self):
@@ -200,3 +213,76 @@ class data_src(object):
     pass
 
 
+if __name__ == '__main__':
+    from sql import *
+    from sql import _q
+    
+    def def_ds():
+        class users(table):
+            sn = int_f()
+            name = str_f()
+            balance = float_f()
+            pass
+
+        class transactions(table):
+            sn = int_f()
+            user = int_f()
+            value = float_f()
+            date = str_f()
+            pass
+
+        @mquery_fs('value', 'date')
+        def user_trans():
+            u = users()
+            t = transactions()
+            return (u * t).fields(t.value, t.date).\
+                where((u.sn == t.user) & (u.name == _q))
+
+        @squery_fs('balance')
+        def user_balance():
+            u = users()
+            return vview(u).fields(u.balance).where(u.name == _q)
+
+        class testds(data_src):
+            ds_definition = ds_define
+            pass
+        return testds
+
+    class dummy_cursor(object):
+        def execute(self, *args):
+            print
+            print 'EXECUTE:'
+            print args[0]
+            if len(args) == 2:
+                print 'ARGS:'
+                print args[1]
+                pass
+            pass
+
+        def fetchall(self, *args):
+            return []
+
+        def fetchone(self, *args):
+            return [1]
+        pass
+
+    class dummy_cx(object):
+        def commit(self):
+            pass
+        def rollback(self):
+            pass
+        def cursor(self):
+            return dummy_cursor()
+        pass
+
+    cx = dummy_cx()
+    testds = def_ds()
+    ds = testds(cx)
+    print ds.users.schema
+    print ds.transactions.schema
+    ds.init_db()
+    ds.user_trans('test')
+    ds.user_balance('test')
+    ds.insert(ds.users, sn=1, name='user1', balance=0)
+    ds.delete(ds.users, ds.users.sn == 333)
+    pass
