@@ -32,6 +32,9 @@ class operand(object):
     
     def get_sql_name_no_alias(self):
 	raise NotImplementedError()
+
+    def get_const_values(self):
+        return ()
     
     def __str__(self):
 	return self.op
@@ -93,6 +96,13 @@ class complex_operand(operand):
 	    return deepcopy(child)
 	new_operand.children = [only_dup_complex(child) for child in self.children]
 	return new_operand
+    
+    def get_const_values(self):
+        consts = ()
+        for child in self.children:
+            consts = consts + child.get_const_values()
+            pass
+        return consts
     pass
 
 def normalize_operand(o):
@@ -110,6 +120,9 @@ class op(object):
 	return self.op_name
     
     get_sql_name_no_alias = get_sql_name
+
+    def get_const_values(self):
+        return ()
     
     def __str__(self):
 	return self.op_name
@@ -118,7 +131,7 @@ class op(object):
 class func_generic(complex_operand):
     def __init__(self, *children):
 	super(func_generic, self).__init__()
-	self.children = children
+	self.children = [normalize_operand(child) for child in children]
 	pass
     
     def get_sql_name(self):
@@ -142,9 +155,22 @@ class isnull(func_generic):
 	arg_str = self.children[0].get_sql_name()
 	return arg_str + ' ISNULL'
     
-    get_sql_name_no_alias = get_sql_name
+    def get_sql_name_no_alias(self):
+	arg_str = self.children[0].get_sql_name_no_alias()
+	return arg_str + ' ISNULL'
     pass
 
+class like(func_generic):
+    def get_sql_name(self):
+        children = self.children
+        return '(' + children[0].get_sql_name() + ' like ' + \
+            children[1].get_sql_name() + ')'
+
+    def get_sql_name_no_alias(self):
+        children = self.children
+        return '(' + children[0].get_sql_name_no_alias() + ' like ' + \
+            children[1].get_sql_name_no_alias() + ')'
+
 class expr(complex_operand):
     def __init__(self, ex=None):
 	super(expr, self).__init__()
@@ -171,7 +197,7 @@ class expr(complex_operand):
     
     def get_sql_name_no_alias(self):
 	return '(' + string.join(map(lambda x: x.get_sql_name_no_alias(), self.children), ' ') + ')'
-    
+
     def __str__(self):
 	return self.get_sql_name_no_alias()
     pass
@@ -197,7 +223,11 @@ class const(operand):
 	    return repr(self.value)
 	return 'NULL'
     
-    get_sql_name_no_alias = get_sql_name
+    def get_sql_name_no_alias(self):
+        return '?'
+
+    def get_const_values(self):
+        return (self.value,)
     
     def __str__(self):
 	return repr(self.value)
@@ -230,6 +260,9 @@ class free_var(sys_sym):
     It is a place holder waiting for user code specifing
     data when executing the query.'''
     sym = '?'
+
+    def get_const_values(self):
+        return (self,)
     pass
 _q = free_var()
 
@@ -490,10 +523,13 @@ class table(object):
 	assigns = self._values_2_assigns(values)
 	a_str = join(assigns, ', ')
 	cmd = 'update %s set %s' % (self.tab_name, a_str)
+        origin_values = [normalize_operand(v) for k, v in values.kv_pairs]
+        args = tuple([v.value for v in origin_values if isinstance(v, const)])
 	if cond:
 	    cmd = cmd + ' where ' + cond.get_sql_name_no_alias()
+            args = args + cond.get_const_values()
 	    pass
-	return cmd
+	return cmd, args
     
     def gen_update_from_vview_cmd(self, values, fields=None, where=None):
 	raise NotImplementedError()
@@ -501,9 +537,11 @@ class table(object):
     def gen_insert_cmd(self, values):
 	from string import join
 	_keys = string.join([getattr(self, k).get_sql_name_no_alias() for k, v in values.kv_pairs], ', ')
-	_values = string.join([normalize_operand(v).get_sql_name_no_alias() for k, v in values.kv_pairs], ', ')
+        origin_values = [normalize_operand(v) for k, v in values.kv_pairs]
+	_values = string.join([v.get_sql_name_no_alias() for v in origin_values], ', ')
 	cmd = 'insert into %s (%s) values(%s)' % (self.tab_name, _keys, _values)
-	return cmd
+        args = tuple([v.value for v in origin_values if isinstance(v, const)])
+	return cmd, args
     
     def gen_delete_cmd(self, where=None):
 	from string import join
@@ -511,7 +549,8 @@ class table(object):
 	if where:
 	    cmd = cmd + ' where ' + where.get_sql_name_no_alias()
 	    pass
-	return cmd
+        args = where.get_const_values()
+	return cmd, args
     
     def __mul__(self, other):
 	return vview(self, other)
@@ -700,5 +739,7 @@ if __name__ == '__main__':
     print t1.factory(f1=_q, f2='aa', f3=11.5).gen_update_cmd(t1.f2==_q)
     print 'Gen_insert_cmd ============================'
     print t1.factory(f1=1, f2='aa', f3=11.5).gen_insert_cmd()
+    print 'Gen_delete_cmd ============================'
+    print t1.gen_delete_cmd((t1.f1 == 3) & like(t1.f2, '%pattern%'))
     pass
 
